
/* 
   softsusy.h 1.0
   Ben Allanach 27.02.2001
      
   Header file for RP conserving MSSM object including all (real) soft SUSY
   breaking parameters and (real) SUSY couplings.
   
   MssmSoftsusy is an object derived from MssmSusy. 
   It therefore doesn't contain a MssmSusy object susypars, hence the susy
   and softsusy parts have the same loop and thresholds parameters.
   All display and set commands involved with running to
   different energy scales have been made more explicitly named.
   
   Tadpoles are stored, they have set and display member
   functions. Full one-loop tadpole calculation implemented. Approximate (but
   finite+log) one-loop corrections for all particles a la Matchev and
   Pierce. Gauge couplings have full one-loop corrections.
   lowOrg changed - no longer inputs a SUSY object, it calculates this
   itself. For this, it uses a new routine: "guessAtSusyMt" in order to get
   the first iteration of the SUSY object
   
   sin thetaW now calculated in DRbar scheme and ALL gauge couplings receive 
   correct one-loop corrections. sinthetaW(DRbar) is stored in sPhysical.
   It's included as a data object in softsusy objects.
   added routine printLong() to provide fuller information in a printout.
   sProblem class added to deal with problems such as lack of rewsb, for
   example. 
   
   Tachyons now flagged by negative masses for scalars in
   printShort. mu is nan in printShort if there's not sufficient convergence.
   Nonperturbative problem flag for if one hits the Landau pole of one
   of the Yukawa couplings.
   
   being more careful about scale dependence. Tadpole variable names, higgs
   vev, realMin changed
   Also, for public version, fineTune and ufb3sl member functions are removed
   from the iteration process.
   */

#ifndef SOFTSUSYH
#define SOFTSUSYH

/* Contains all supersymmetric MSSM parameters INCLUDING SOFT TERMS and RGEs
   for interpolating them (In MSSM domain) */
#include<iostream.h>
#include<math.h>
#include<fstream.h>
#include "lowe.h"
#include "susy.h"
#include "def.h"
#include "linalg.h"
#include "complex.h"

// Masses of the physical particles etc: We have made assumption that scalar
// mixing is small except in third generation and Higgs sector
struct sPhysical
{
  // in order: h^0, A^0, H^0, H^+-
  DoubleVector mhiggs;
  // sneutrino masses
  DoubleVector msnu;
  // chargino/neutralino masses: not ordered
  DoubleVector mch, mneut;
  double mGluino;
  // neutralino mixing
  DoubleMatrix mixNeut;
  // chargino and third famliy mixing angles
  double thetaL, thetaR, thetat, thetab, thetatau;
  // sparticle masses in order (i=L/R, family)
  DoubleMatrix mu, md, me;
  // Higgs mixing angle (alpha)
  double thetaH;
  
  // DRbar quantities evaluated at MSusy
  double t1OV1Ms, t2OV2Ms;
  
  // Feynman rules
  DoubleMatrix aChi0ChicW, bChi0ChicW;
  
  sPhysical();
  sPhysical(const sPhysical &);
  const sPhysical & operator = (const sPhysical &);
}; 

ostream & operator <<(ostream &, const sPhysical &); 

typedef enum {mQl=1, mUr, mDr, mLl, mEr} softMasses;
typedef enum {UA=1, DA, EA} trilinears;

struct sProblem
{
  bool noConvergence;
  bool tachyon;
  bool muSqWrongSign;
  bool b;
  bool higgsUfb;
  bool nonperturbative;
  bool test() const 
  {return (noConvergence || tachyon || muSqWrongSign ||
	   higgsUfb || nonperturbative);};
  
  sProblem();
  //  void displayverbose(ostream &);
  inline sProblem(const sProblem &);
  const sProblem & operator = (const sProblem &);
  
};
ostream & operator <<(ostream &st, const sProblem & p);
istream & operator >>(istream & left, sPhysical &s);
class MssmSoftsusy;
istream & operator >>(istream &left, MssmSoftsusy &s);
//  external object temp used to get objects into external routines, however:
//  don't use it!
static MssmSoftsusy *tempSoft;

/* ------------------SOFTSUSY starts here ---------------------*/
class MssmSoftsusy: public MssmSusy
{
private:
  QedQcd dataSet; // contains low energy data on quark masses etc
  DoubleVector mGaugino; // Gauginio masses, see ::beta for definitions
  DoubleMatrix ua, da, ea; // Trilinear soft terms..
  DoubleMatrix mQLsq, mURsq, mDRsq, mLLsq, mSEsq; // soft mass squared matrices
  double m32; // Gravitino mass
  double higgsVevMs; // around 246.22 
  double b, mH1sq, mH2sq; // Bilinear Higgs soft parameters
  
  sPhysical physpars;
  double minV;
  sProblem problem;
public:
  MssmSoftsusy();
  MssmSoftsusy(const MssmSusy &);
  MssmSoftsusy(const MssmSoftsusy &);
  MssmSoftsusy(const MssmSusy &, const sPhysical &, const DoubleVector &,
		const DoubleMatrix &, const
		DoubleMatrix &, const DoubleMatrix &, const DoubleMatrix &,
		const DoubleMatrix &, const DoubleMatrix &, const
		DoubleMatrix &, const DoubleMatrix &, double, double, 
		double, double, int, int, int, double, double);
  const MssmSoftsusy & operator=(const MssmSoftsusy & s);
  
  sProblem displayProblem() const {return problem; };
  inline QedQcd displayDataSet() const;
  inline double displayHiggsVevMs() const;
  double displayGravitino() const;
  DoubleMatrix displayTrilinear(trilinears) const;
  double displaySoftA(trilinears, int, int) const;
  DoubleMatrix displaySoftMassSquared(softMasses) const;
  int displayMinimum() const;
  inline double displayB() const;
  inline double displayMh1Squared() const;
  inline double displayMh2Squared() const;
  inline DoubleVector displayGaugino() const;
  inline MssmSusy displaySusy() const;
  inline sPhysical displayPhys() const;
  virtual DoubleVector display() const;
  double displayMinpot() const;
  double displayTadpole1Ms() const;
  double displayTadpole2Ms() const;
  
  void flagNonperturbative(bool a) { problem.nonperturbative = a; };
  void flagTachyon(bool a) { problem.tachyon = a; };
  void flagB(bool a) { problem.b = a; };
  void flagNoconvergence(bool a) { problem.noConvergence = a; };
  void flagMusqwrongsign(bool a) { problem.muSqWrongSign = a; };
  void flagHiggsufb(bool a) { problem.higgsUfb = a; };
  void flagAllProblems(bool a) { problem.tachyon = a; problem.b = a;
  problem.noConvergence = a; problem.higgsUfb = a;
  problem.nonperturbative = a; }
  
  void setHiggsVevMs(double v) { higgsVevMs = v; };
  void setData(const QedQcd & r) { dataSet = r; };
  void setM32(double);
  void setMinpot(double);
  void doUfb3(double mgut);
  void setSoft(const MssmSoftsusy &);
  void setSoftMassElement(softMasses, int, int, double);
  void setSoftMassMatrix(softMasses, const DoubleMatrix &);
  void setTrilinearMatrix(trilinears, const DoubleMatrix &);
  void setTrilinearElement(trilinears k, int i, int j, double a);
  void setGauginoMass(int, double);
  void setAllGauginos(const DoubleVector &);
  void setB(double);
  void setMh1Squared(double);
  void setMh2Squared(double);
  virtual void set(const DoubleVector &);
  void setSusy(const MssmSusy &);
  void setPhys(const sPhysical & s) { physpars = s; };
  virtual void setTadpole1Ms(double mt, double sinthDRbar);
  virtual void setTadpole2Ms(double mt, double sinthDRbar);
  void overwriteTadpole1Ms(double value){physpars.t1OV1Ms = value;};
  void overwriteTadpole2Ms(double value){physpars.t2OV2Ms = value;};
  
  virtual DoubleVector beta() const; 
  MssmSoftsusy beta2() const; 
  
  void universal(double,  double,  double,  double, double);
  
  void addAmsb();
  void standardSugra(double, double, double);
  void universalScalars(double m0);
  void universalGauginos(double m12);
  void universalTrilinears(double a0);
  
  
  DoubleMatrix treeSpartDoubleMatrix(double mSm, softMasses rsm, int family, double
			 pizztMS, double sinthMS) const; 
  DoubleMatrix addSpartLoop(DoubleMatrix mass, double mSm, softMasses
			rsm, int family) const;
  DoubleMatrix getSpartDoubleMatrix(double mSm, softMasses rsm, int family, int
			  accuracy, double pizztMS, double sinthMS) const;
  
  virtual DoubleVector getSpart(double, softMasses, double &, int, int, double
			  pizztMS, double sinthMS);
  
  double lambda(double theta, double beta) const;
  double omega(double theta, double beta) const;  
  double lambdab(double theta, double beta) const;
  double omegab(double theta, double beta) const;  
  
  void sparticleThresholdCorrections(double tb);
  double qedSusythresh(double alphaEm, double Q) const;
  double qcdSusythresh(double alphasMSbar, double Q) const;
  double calcMs() const;
  void physical(int accuracy);
  double calcRunningMt() const;
  double calcRunningMtau() const;  
  double calcRunningMb() const;
  double calcHt(double vev) const;
  double calcHb(double vev) const;
  double calcHtau(double vev) const;
  double calcSinthdrbar() const;
  double getVev() const;
  virtual void charginos(int accuracy, double piwwt);
  virtual void addCharginoLoop(DoubleMatrix &);
  virtual void neutralinos(int accuracy, double piwwt, double pizzt);
  virtual void addNeutralinoLoop(DoubleMatrix &);
  void gluino();
  virtual void higgs(int, double piwwt, double pizzt);
  int rewsbMu(int, double &) const;
  int rewsbB(double, double &) const;
  void rewsb(int, double);
  void rewsbTreeLevel(int);
  void iterateMu(double & muold, double & munew, int sgnMu, double mt, 
		  int maxTries, double pizztMS, double sinthDRbar, double tol,
		 int  & err);
  double predTanb() const;
  double predMzsq(double &) const;
  DoubleVector fineTune(void (*boundaryCondition)(MssmSoftsusy &, const
					      DoubleVector &), 
		   const DoubleVector & bcPars, double MX) const;
  double it1par(int numPar, const DoubleVector & bcPars) const;
  double ufb3sl(double) const;
  double realMinMs() const;
  
  double piZZT(double p, double Q) const;
  double piWWT(double p, double Q) const;
  //  double piZGT(double p, double Q) const;
  double piAA(double p, double Q) const;
  void rhohat(double & inrho, double & insin, double & outrho,
	      double & outsin, double alphaMZDRbar, double tol, int maxTries) 
    const;
  
  int lsp(double &, int &, int &) const;
  
  void printShort(const DoubleVector & ft) const;
  void printLong();
  
  double thet(double a, double b, double c);
 
  void mpzNeutralinos(ComplexMatrix & n, DoubleVector & mneut) const; 
  void mpzCharginos(ComplexMatrix & u, ComplexMatrix & v, DoubleVector & mch)
    const;
};

// ---------------------- inline class members ------------------------
inline sPhysical::sPhysical()
  : mhiggs(4), msnu(3), mch(2), mneut(4), mGluino(0.0),
    mixNeut(4, 4), thetaL(0.0), thetaR(0.0), thetat(0.0), thetab(0.0),
    thetatau(0.0), mu(2, 3), md(2, 3), me(2, 3), thetaH(0.0), 
    t1OV1Ms(0.0), t2OV2Ms(0.0), aChi0ChicW(4, 2), bChi0ChicW(4, 2)
{};

inline sPhysical::sPhysical(const sPhysical & s)
  : mhiggs(s.mhiggs), msnu(s.msnu), mGluino(s.mGluino), 
    mch(s.mch), 
    mneut(s.mneut), mixNeut(s.mixNeut), thetaL(s.thetaL),
    thetaR(s.thetaR), thetat(s.thetat), thetab(s.thetab),
    thetatau(s.thetatau), mu(s.mu), md(s.md), me(s.me), thetaH(s.thetaH),
    t1OV1Ms(s.t1OV1Ms), t2OV2Ms(s.t2OV2Ms),
    aChi0ChicW(s.aChi0ChicW), bChi0ChicW(s.bChi0ChicW)
{};

inline sProblem::sProblem()
  : noConvergence(false), tachyon(false), muSqWrongSign(false),
    b(false), higgsUfb(false), nonperturbative(false)
{}

inline sProblem::sProblem(const sProblem & s)
  : noConvergence(s.noConvergence), tachyon(s.tachyon),
    muSqWrongSign(s.muSqWrongSign), b(s.b),
    higgsUfb(s.higgsUfb), nonperturbative(s.nonperturbative)
{}

inline MssmSoftsusy::MssmSoftsusy()
  : MssmSusy(), physpars(), mGaugino(3), ua(3, 3), da(3, 3), ea(3, 3), 
    mQLsq(3, 3), problem(), 
    mURsq(3, 3), mDRsq(3, 3), mLLsq(3, 3), mSEsq(3, 3), b(0.0), mH1sq(0.0), mH2sq(0.0),
    minV(6.66e66), m32(0.0), higgsVevMs(246.22), dataSet() {
      setPars(110);
      setMu(0.0);
      setLoops(0);
      setThresholds(0);
}

inline MssmSoftsusy::MssmSoftsusy(const MssmSoftsusy & s)
  : MssmSusy(s.displaySusy()), physpars(s.displayPhys()),
    mGaugino(s.displayGaugino()), ua(s.displayTrilinear(UA)),
    da(s.displayTrilinear(DA)), 
    ea(s.displayTrilinear(EA)), mQLsq(s.displaySoftMassSquared(mQl)),
    mURsq(s.displaySoftMassSquared(mUr)), 
    mDRsq(s.displaySoftMassSquared(mDr)),
    mLLsq(s.displaySoftMassSquared(mLl)),
    mSEsq(s.displaySoftMassSquared(mEr)), 
    b(s.displayB()), mH1sq(s.displayMh1Squared()),
    mH2sq(s.displayMh2Squared()),
    problem(s.problem), minV(s.minV), m32(s.m32),
    higgsVevMs(s.displayHiggsVevMs()), dataSet(s.displayDataSet()) {
      setPars(110);
      setMu(s.displayMu()); 
      setLoops(s.displayLoops());
      setThresholds(s.displayThresholds());
}

inline MssmSoftsusy::MssmSoftsusy(const MssmSusy &s)
  : MssmSusy(s), physpars(), mGaugino(3), ua(3, 3), da(3, 3), ea(3, 3), 
    mQLsq(3, 3), mURsq(3, 3), mDRsq(3, 3), mLLsq(3, 3), mSEsq(3, 3), b(0.0),
    mH1sq(0.0),  mH2sq(0.0), problem(), minV(6.66e66), m32(0.0),
    higgsVevMs(246.22), dataSet() { 
      setPars(110);
      setMu(s.displayMu()); 
      setLoops(s.displayLoops());
      setThresholds(s.displayThresholds());
}

inline MssmSoftsusy::MssmSoftsusy(const MssmSusy & s, const sPhysical &
				    sp, const DoubleVector & mG, const DoubleMatrix & aU,
				    const DoubleMatrix & aD, const DoubleMatrix &
				    aE, const DoubleMatrix & mQl, const
				    DoubleMatrix & mUr, const DoubleMatrix & mDr,
				    const DoubleMatrix & mLl, const DoubleMatrix &
				    mEr, double bSoft, double mH1sq, double
				    mH2sq, double mu, int l, int t, int sc,
				    double mg, double hv)
  : MssmSusy(s), physpars(sp), mGaugino(mG), ua(aU), da(aD), ea(aE),
    mQLsq(mQl), mURsq(mUr), mDRsq(mDr), mLLsq(mLl), mSEsq(mEr), b(bSoft),
    mH1sq(mH1sq), mH2sq(mH2sq), problem(), minV(6.66e66), m32(mg),
    higgsVevMs(hv), dataSet() {
      setPars(110);
      setMu(mu);
      setLoops(l);
      setThresholds(t);
}

inline QedQcd MssmSoftsusy::displayDataSet() const { return dataSet; }

inline double MssmSoftsusy::displayHiggsVevMs() const { 
  return higgsVevMs; 
}

inline double MssmSoftsusy::displayGravitino() const { return m32; }

inline double MssmSoftsusy::displayB() const { return b; }

inline double MssmSoftsusy::displayMh1Squared() const { return mH1sq; }

inline double MssmSoftsusy::displayMh2Squared() const { return mH2sq; }

inline DoubleVector MssmSoftsusy::displayGaugino() const { return mGaugino; }

inline MssmSusy MssmSoftsusy::displaySusy() const { 
  return MssmSusy(displayYukawaMatrix(YU), displayYukawaMatrix(YD),
		   displayYukawaMatrix(YE), displayGauge(),
		   displaySusyMu(), displayTanb(), displayMu(),
		   displayLoops(), displayThresholds()); 
}

inline sPhysical MssmSoftsusy::displayPhys() const { return physpars; }

inline double MssmSoftsusy::displayMinpot() const { return minV; } 

inline double MssmSoftsusy::displayTadpole1Ms() const {
  return physpars.t1OV1Ms; 
}

inline double MssmSoftsusy::displayTadpole2Ms() const {
  return physpars.t2OV2Ms; 
}

inline void MssmSoftsusy::setM32(double a) { m32 = a; }

inline void MssmSoftsusy::setGauginoMass(int i, double f) { 
  mGaugino(i) = f; 
}

inline void MssmSoftsusy::setMinpot(double f) { minV = f; }
inline void MssmSoftsusy::doUfb3(double mgut) { setMinpot(ufb3sl(mgut) -
							  realMinMs()); } 
inline void MssmSoftsusy::setB(double f) { b = f; }
inline void MssmSoftsusy::setMh1Squared(double f) { mH1sq = f; }
inline void MssmSoftsusy::setMh2Squared(double f) { mH2sq = f; }

// ---------------------- Other functions associated with class -------------
void printShortInitialise();
ostream & operator <<(ostream &, const MssmSoftsusy &); 
void itLowsoft(MssmSoftsusy & in, MssmSoftsusy & out, int maxTries, double
		MX, int sgnMu, double tol, double tanb, void
		(*boundaryCondition)(MssmSoftsusy &, const DoubleVector &),
		const DoubleVector & pars);
MssmSoftsusy lowOrg(void (*boundaryCondition)(MssmSoftsusy &, const
					      DoubleVector &),
 double mx, const DoubleVector & pars, int sgnMu, double tanb, const QedQcd &
 oneset, int thresholds, double epsilon);
double sumTol(const MssmSoftsusy & in, const MssmSoftsusy & out);
inline double ccbSqrt(double);
void recogLsp(int, int);
double rho2(double r);
MssmSusy guessAtSusyMt(double tanb, const QedQcd & oneset);

// ufb3 direction CCB calculations
Complex fnfn(double x);
double integrandThreshbnr(double x);
extern double minimufb3(double);
double ufb3fn(double,double,double,int,MssmSoftsusy);
double getQhat(double, double, double, double, double, MssmSoftsusy &);

// For calculation of PV functions
double integrandThreshbn(double x);
double bIntegral(int n, double p, double m1, double m2, double mt);
DoubleVector dd(double x, const DoubleVector & y);

// Passarino-Veltman function definitions
double b0(double p, double m1, double m2, double q);
double b1(double p, double m1, double m2, double q);
double b22(double p,  double m1, double m2, double q);
double c0(double m1, double m2, double m3);
double d27(double m1, double m2, double m3, double m4);
double d0(double m1, double m2, double m3, double m4);

// inlined PV functions
inline double a0(double m, double q) {
  if (m == 0.0) return 0.0;
  return sqr(m) * (1.0 - log(sqr(m / q)));
}

inline double ffn(double p, double m1, double m2, double q) {
  return a0(m1, q) - 2 * a0(m2, q) - (2 * sqr(p) + 2 * sqr(m1) - 2 * sqr(m2)) *
    b0(p, m1, m2, q);
}

inline double gfn(double p, double m1, double m2, double q) {
  return (sqr(p) - sqr(m1) - sqr(m2)) * b0(p, m1, m2, q) - a0(m1, q) 
    - a0(m2, q); 
}

inline double hfn(double p, double m1, double m2, double q) {
  return 4.0 * b22(p, m1, m2, q) + gfn(p, m1, m2, q);
}

inline double b22bar(double p, double m1, double m2, double q) {
  return b22(p, m1, m2, q) - 0.25 * a0(m1, q) - 0.25 * a0(m2, q);
}

inline double fB(const Complex & x) {
  return (log(1.0 - x) - x * log(1.0 - 1.0 / x) - 1.0).real();
};

#endif



